@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: arg.c 900 2008-01-05 01:13:26Z hubert@u.washington.e
 
 /*
  * ========================================================================
+ * Copyright 2013-2015 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -58,6 +58,10 @@ static char args_err_missing_aux[] =		N_("missing argument for option \"-aux\"")
 #ifdef	PASSFILE
 static char args_err_missing_passfile[] =	N_("missing argument for option \"-passfile\"");
 static char args_err_non_abs_passfile[] =	N_("argument to \"-passfile\" should be fully-qualified");
+#ifdef SMIME
+static char args_err_missing_pwdcertdir[] =	N_("missing argument for option \"-pwdcertdir\"");
+static char args_err_non_abs_pwdcertdir[] =	N_("argument to \"-pwdcertdir\" should be fully-qualified");
+#endif /* SMIME inside PASSFILE */
 #endif
 static char args_err_missing_sort[] =		N_("missing argument for option \"-sort\"");
 static char args_err_missing_flag_arg[] =	N_("missing argument for flag \"%c\"");
@@ -130,6 +134,10 @@ N_(" -erase_stored_passwords\tEliminate any stored passwords"),
 #ifdef	PASSFILE
 N_(" -passfile <fully_qualified_filename>\tSet the password file to something other"),
 N_("\t\tthan the default"),
+#ifdef SMIME
+N_(" -pwdcertdir <fully_qualified_path>\tSet the directory to store a personal"),
+N_("\t\tkey and certificate to encrypt and decrypt your password file."),
+#endif /* SMIME inside PASSFILE */
 #endif	/* PASSFILE */
 
 #ifdef	LOCAL_PASSWD_CACHE
@@ -290,6 +298,31 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
+#ifdef SMIME
+	      else if(strcmp(*av, "pwdcertdir") == 0){
+		  if(--ac){
+		      if((str = *++av) != NULL){
+			  if(!is_absolute_path(str)){
+			      display_args_err(_(args_err_non_abs_pwdcertdir),
+					       NULL, 1);
+			      ++usage;
+			  }
+			  else{
+			      if(pine_state->pwdcertdir)
+				fs_give((void **)&pine_state->pwdcertdir);
+
+			      pine_state->pwdcertdir = cpystr(str);
+			  }
+		      }
+		  }
+		  else{
+		      display_args_err(_(args_err_missing_pwdcertdir), NULL, 1);
+		      ++usage;
+		  }
+
+		  goto Loop;
+	      }
+#endif  /* SMIME inside PASSFILE */
 #endif	/* PASSFILE */
 
 #ifdef  LOCAL_PASSWD_CACHE
@@ -1157,13 +1190,21 @@ pinerc_cmdline_opt(char *arg)
     if(!arg || !arg[0])
       return 0;
 
-    for(v = ps_global->vars; v->name != NULL; v++){
-      if(v->is_used && struncmp(v->name, arg, strlen(v->name)) == 0){
-	  p1 = arg + strlen(v->name);
+    if((value = strchr(arg, '=')) != NULL){
+	i = value - arg;
+	arg[i] = '\0';
+    }
+    else
+ 	i = -1;
 
-	  /*----- Skip to '=' -----*/
+    for(v = ps_global->vars; v->name != NULL; v++){
+	if(v->is_used && strucmp(v->name, arg) == 0){
+	  p1 = arg + strlen(v->name);
+	  if(i > 0) arg[i] = '=';
+
+	    /*----- Skip to '=' -----*/
 	  while(*p1 && (*p1 == '\t' || *p1 == ' '))
-	    p1++;
+	     p1++;
 
 	  if(*p1 != '='){
 	      char buf[MAILTMPLEN];
@@ -1175,8 +1216,11 @@ pinerc_cmdline_opt(char *arg)
 
 	  p1++;
           break;
-      }
+	}
     }
+	
+
+    if(i > 0) arg[i] = '=';
 
     /* no match, check for a feature name used directly */
     if(v->name == NULL){
