@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: ldap.c 1204 2009-02-02 19:54:23Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013 Eduardo Chappa
+ * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -1700,7 +1700,7 @@ our_ldap_set_option(LDAP *ld, int option, void *optdata)
 	break;
     
       default:
-	panic("LDAP function not implemented");
+	alpine_panic("LDAP function not implemented");
     }
 #endif
 
