@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: mailcap.c 1012 2008-03-26 00:44:22Z hubert@u.washing
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013 Eduardo Chappa
+ * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -248,7 +248,7 @@ mc_process_file(char *file)
 	return;
 
       default:
-	panic("Programmer botch in mc_process_file");
+	alpine_panic("Programmer botch in mc_process_file");
 	/*NOTREACHED*/
     }
 
