@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: save.c 1204 2009-02-02 19:54:23Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2009 University of Washington
- * Copyright 2013 Eduardo Chappa
+ * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -240,7 +240,7 @@ save_get_fldr_from_env(char *fbuf, int nfbuf, ENVELOPE *e, struct pine *state,
 	break;
       
       default:
-	panic(botch);
+	alpine_panic(botch);
 	break;
     }
 
